// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

///////////////////////////////
// two definitions as given in the assignment
#define N 10     // number of sylinders
#define G 20.0f  // slope in degres

// a few other handy ones
#define PI 3.1415f
#define Gr G*(PI/180.0f) // G in radians

#define D 10.0f          // meters between cylinders
#define H 3.0f           // height of cylinferd
#define _WIDTH 2.5f*D    // meters across drawn slope
#define _LENGTH N*D      // meters along slope

#define TIMERID 3        // any timer id


///////////////////////////////////////////
// function that calclates spehere(skiers)
// position for any meaningfull parameter, t
// in [0..N/2]
BOOL GetPos(float t, float &x,float &y,float &z)
{
	// the whole journey takes t=N/2
  // (two cylinders in one sinus cycle)
	// Returns TRUE if t is (still) legal
  BOOL ret=TRUE;

  if(t<0.0f)
	{
	 // job is done
		t=0.0f;
		ret=FALSE;
	}
  else if(t>N*0.5f)
  {
		// bigger than max, reduce
		t=N*0.5f;
    ret=FALSE;
  }

	// calc
  float l=2*D*t; // length on slope
	x=l*(float)cos(Gr);
	y=D*(float)sin(2*t*PI);
	z=l*(float)sin(Gr)+H/2.0f;
	return ret;
}

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_COMMAND(ID_ANIMATE_STARTSTOP, OnAnimateStartstop)
	ON_COMMAND(ID_VIEW_ASASKIIER, OnViewAsaskiier)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ASASKIIER, OnUpdateViewAsaskiier)
	ON_COMMAND(ID_VIEW_ASAAEROPLANE, OnViewAsaaeroplane)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ASAAEROPLANE, OnUpdateViewAsaaeroplane)
	ON_COMMAND(ID_VIEW_FOG, OnViewFog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FOG, OnUpdateViewFog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	// initialize everything
  m_xv=0.0f;
	m_zv=0.0f;
	m_s=0.0f;
	
	m_eye_x=2*_LENGTH;
	m_eye_y=_WIDTH/2.0f;
	m_eye_z=_WIDTH;

	m_sphere_x=m_sphere_y=m_sphere_z=0.0f;

	m_Is_Animating=FALSE;
  m_Is_Skier=FALSE;
  m_Is_Fog=FALSE;

  m_dt=0.05f; // parameter increment on animation

  m_hRC=NULL;
	m_hDC=NULL;

};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	// Clear the color and depth buffers.
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	////////////////////////
  // skier or not
  if(m_Is_Skier)
  {
     // position the eye as a skiers eye
     // looking at the end of the slope
	   float x,y,z;
     GetPos(m_t,x,y,z);
     gluLookAt(x,y,z,	  // eye
				       0.0f,0.0f,0.0f,	// end of slope
				       0.0f,0.0f,1.0f);	// z is up
    
     // set up headlight
	   GLfloat position[] = {x,y,z+D*0.5f,0.0f };
     glLightfv(GL_LIGHT0, GL_POSITION, position);

  }
  else
  {
   // we have a fixed eyepos and
   // rotate the scene according to mousemoves
   m_eye_x=0.0f;
	 m_eye_y=0.0f;
	 m_eye_z=_LENGTH;
   
   gluLookAt(m_eye_x,m_eye_y,m_eye_z,	// eye
				     _LENGTH/2.0f,_WIDTH/2.0f,0.0f,	// looking at
             0.0f,0.0f,1.0f);	// z is up
   
   // set up headlight
   GLfloat position[] = {m_eye_x,m_eye_y,m_eye_z,1.0f };
   glLightfv(GL_LIGHT0, GL_POSITION, position);
 
   // handle interactive movement
   glTranslatef(_LENGTH/4.0f,0.0f,0.0f);
	 glRotatef(m_zv,0.0f,0.0f,1.0f);
	 glRotatef(m_xv,0.0f,1.0f,0.0f);
   glTranslatef(-_LENGTH/4.0f,0.0f,0.0f);
 
	}
  

  ///////////////////////////////
  // Fog
	if(m_Is_Fog)
	{
		///////////////////////////////
		// fog
		GLfloat fog_col[4]={1.0f,1.0f,1.0f,0.0f};
		glEnable(GL_FOG);
		glFogf(GL_FOG_MODE,GL_LINEAR);
		glFogfv(GL_FOG_COLOR,fog_col);
		glFogf(GL_FOG_END,10.0f*D);
		glFogf(GL_FOG_START,3.0f*D);
	}
	else
		glDisable(GL_FOG);

	
	////////////////////////////////
	// Draw the horizontal reference
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.0f,-_WIDTH/2.0f,0.0f);
		glVertex3f(0.0f,_WIDTH/2.0f,0.0f);
		glVertex3f(_LENGTH*(float)cos(Gr),_WIDTH/2.0f,0.0f);
		glVertex3f(_LENGTH*(float)cos(Gr),-_WIDTH/2.0f,0.0f);
	glEnd();

	////////////////////////////////
	// Draw the slope
	CMaterial::SetMaterial(WARM_WHITE);
	glBegin(GL_POLYGON);
    glNormal3f(0.0f,0.0f,(float)cos((90-G)*PI/180.0f));
    glVertex3f(0.0f,-_WIDTH/2.0f,0.0f);
		glVertex3f(0.0f,_WIDTH/2.0f,0.0f);
		glVertex3f(_LENGTH*(float)cos(Gr),_WIDTH/2.0f,_LENGTH*(float)sin(Gr));
		glVertex3f(_LENGTH*(float)cos(Gr),-_WIDTH/2.0f,_LENGTH*(float)sin(Gr));
	glEnd();

	GLUquadricObj *glpQ=gluNewQuadric();

	///////////////////////////////////
	// Draw the sylinders
	CMaterial::SetMaterial(RED_PLASTIC);
	for(int ix=0;ix<N;ix++)
	{
		glPushMatrix();
		glTranslatef(D*(ix+0.5f)*(float)cos(Gr),0.0f,D*(ix+0.5f)*(float)sin(Gr));
		gluCylinder(glpQ,0.2f,0.2f,H,10,2);
		glPopMatrix();
	}

	if(!m_Is_Skier)
  {
     /////////////////////////////
	   // draw the sphere
	   glTranslatef(m_sphere_x,m_sphere_y,m_sphere_z);
	   gluSphere(glpQ,0.5f,10,10);
  }

	gluDeleteQuadric(glpQ);


	// finnish drawing
	glFlush();
}


void CStdView::InitializeRContext()
{
 glEnable(GL_DEPTH_TEST);
 glDepthFunc(GL_LESS);
    
 // prepare ligthsource
 GLfloat ambient[] = {0.4f,0.4f,0.4f,1.0f };
 GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
 GLfloat position[] = {20.0f,30.0f,10.0f,0.0f };
 GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };

 glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
 glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
 glLightfv(GL_LIGHT0, GL_POSITION, position);
	
 glEnable(GL_LIGHT0);
 glEnable(GL_LIGHTING);

 glEnable(GL_NORMALIZE);
 // smooth the drawing
 glShadeModel(GL_SMOOTH);

 // set background to white
 glClearColor(1.0f, 1.0f, 1.0f, 0.0f);

}

void CStdView::SetPerspective(int cx, int cy)
{
    // We will work on the projection matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
	// set up your viewing volume
	gluPerspective(40.0f, (GLdouble)cx/cy, 0.1f, 1000.0f);
	// reset matrix mode 
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down 
	if(nFlags & MK_LBUTTON)
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2))
		{
			// then we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);
			
			m_zv+=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_xv+=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}


void CStdView::OnTimer(UINT nIDEvent) 
{
	// update spherepos when animating
	if(nIDEvent==TIMERID)
  {
    m_t-=m_dt;
	  if(!GetPos(m_t,m_sphere_x,m_sphere_y,m_sphere_z))
    {
		  KillTimer(TIMERID);
      m_Is_Animating=FALSE;
    }
	  RedrawWindow();
  }
	COGLView::OnTimer(nIDEvent);
}

void CStdView::OnAnimateStartstop() 
{
	if(m_Is_Animating)
	{
		KillTimer(TIMERID);
		m_Is_Animating=FALSE;
	}
	else
	{
		m_Is_Animating=TRUE;
		m_t=N/2.0f;
		SetTimer(TIMERID,100,NULL);
	}
}

void CStdView::OnViewAsaskiier() 
{
	m_Is_Skier=TRUE;
  m_t=N/2.0f;
  RedrawWindow();
}

void CStdView::OnUpdateViewAsaskiier(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Is_Skier);
}

void CStdView::OnViewAsaaeroplane() 
{
	m_Is_Skier=FALSE;
  RedrawWindow();
}

void CStdView::OnUpdateViewAsaaeroplane(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_Is_Skier);
}

void CStdView::OnViewFog() 
{
  m_Is_Fog=!m_Is_Fog;
  RedrawWindow();
}

void CStdView::OnUpdateViewFog(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_Is_Fog);
}
